﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private string gridFilePath = "..\\..\\data\\result.grid";
        private string csvFilePath = "..\\..\\data\\GrayCountyIrrigationWellDepths.csv";
        private Dictionary<PointShape, double> wellDepthPointData;
        ManagedProj4Projection proj;
        private int isoLineLevelCount = 25;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.CurrentExtent = new RectangleShape(-11204955.4870063, 4543351.66721459, -11200273.7190238, 4539757.60327179);
            //create grid file with SphericalMercator projection
            wellDepthPointData = GetPointsData();
            CreateGridFile();

            GoogleMapsOverlay googleMapsOverlay = new GoogleMapsOverlay();

            LayerOverlay overlay = new LayerOverlay();
            overlay.Layers.Add(GetGridIsoLineLayer());
            overlay.Layers.Add(GetGridFeatureLayer());

            winformsMap1.Overlays.Add(googleMapsOverlay);
            winformsMap1.Overlays.Add(overlay);
            winformsMap1.Refresh();

        }

        private GridIsoLineLayer GetGridIsoLineLayer()
        {
            //Load the grid we just created into the GridIsoLineLayer using the number of breaks defines
            //on the screen
            Collection<double> isoLineLevels = GridIsoLineLayer.GetIsoLineLevels(wellDepthPointData, isoLineLevelCount);
            GridIsoLineLayer isoLineLayer = new GridIsoLineLayer(gridFilePath, isoLineLevels);
            //Create a series of colors from blue to red that we will use for the breaks
            Collection<GeoColor> colors = GeoColor.GetColorsInQualityFamily(GeoColor.StandardColors.Blue, GeoColor.StandardColors.Red, isoLineLevels.Count, ColorWheelDirection.Clockwise);

            //Setup a class break style based on the isoline levels and the colors
            ClassBreakStyle classBreakLineStyle = new ClassBreakStyle(isoLineLayer.DataValueColumnName);
            LineStyle firstStyle = new LineStyle(new GeoPen(colors[0], 3));
            classBreakLineStyle.ClassBreaks.Add(new ClassBreak(double.MinValue, firstStyle));
            for (int i = 0; i < colors.Count - 1; i++)
            {
                LineStyle style = new LineStyle(new GeoPen(colors[i + 1], 3));
                classBreakLineStyle.ClassBreaks.Add(new ClassBreak(isoLineLevels[i], style));
            }
            isoLineLayer.CustomStyles.Add(classBreakLineStyle);

            //Create the text styles to label the lines
            TextStyle textStyle = TextStyles.CreateSimpleTextStyle(isoLineLayer.DataValueColumnName, "Arial", 8, DrawingFontStyles.Bold, GeoColor.StandardColors.Black, 0, 0);
            textStyle.OverlappingRule = LabelOverlappingRule.NoOverlapping;
            textStyle.SplineType = SplineType.StandardSplining;
            textStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            textStyle.TextLineSegmentRatio = 9999999;
            textStyle.FittingLineInScreen = true;
            textStyle.SuppressPartialLabels = true;
            isoLineLayer.CustomStyles.Add(textStyle);

            return isoLineLayer;
        }

        private GridFeatureLayer GetGridFeatureLayer()
        {
            //Get the line breaks
            Collection<double> isoLineBreaks = GridIsoLineLayer.GetIsoLineLevels(wellDepthPointData, isoLineLevelCount);

            //Load a new GridFeatureLayer based on the current grid file
            GridFeatureLayer gridFeatureLayer = new GridFeatureLayer(gridFilePath);

            //Create a series of colors from blue to red that we will use for the breaks
            Collection<GeoColor> colors = GeoColor.GetColorsInQualityFamily(GeoColor.StandardColors.Blue, GeoColor.StandardColors.Red, isoLineBreaks.Count, ColorWheelDirection.Clockwise);

            //Create a class break style
            ClassBreakStyle classBreakLineStyle = new ClassBreakStyle(gridFeatureLayer.DataValueColumnName);

            //Setup a class break style based on the isoline levels and the colors
            AreaStyle firstStyle = new AreaStyle(new GeoPen(GeoColor.FromArgb(50, colors[0]), 1), new GeoSolidBrush(GeoColor.FromArgb(50, colors[0])));
            classBreakLineStyle.ClassBreaks.Add(new ClassBreak(double.MinValue, firstStyle));
            for (int i = 1; i < colors.Count - 1; i++)
            {
                AreaStyle style = new AreaStyle(new GeoPen(GeoColor.FromArgb(50, colors[i]), 1), new GeoSolidBrush(GeoColor.FromArgb(50, colors[i])));
                classBreakLineStyle.ClassBreaks.Add(new ClassBreak(isoLineBreaks[i], style));
            }

            gridFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(classBreakLineStyle);
            gridFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            return gridFeatureLayer;
        }

        private void CreateGridFile()
        {
            //Get the current extent since we use that to gerenate the grid.  Of course this is just for
            //the demo and in the real world you can use any extent you like
            RectangleShape currentDrawingExtent = ExtentHelper.GetDrawingExtent(winformsMap1.CurrentExtent, winformsMap1.Width, winformsMap1.Height);

            //Calculate the cell size based on how many rows and columns you specified
            double cellSize = Math.Min(currentDrawingExtent.Width / 100, currentDrawingExtent.Height / 100);

            //Greate the grid definition based on the extent, cell size etc.
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, cellSize, -9999, wellDepthPointData);

            FileStream gridFileStream = null;
            try
            {
                gridFileStream = new FileStream(gridFilePath, FileMode.Create, FileAccess.ReadWrite);

                //Generate the grid based on Inverse Distance Weighted interpolation model.  You can define your own model if needed.
                GridFeatureSource.GenerateGrid(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(2, double.MaxValue), gridFileStream);
            }
            finally
            {
                if (gridFileStream != null) { gridFileStream.Close(); }
            }
        }

        private Dictionary<PointShape, double> GetPointsData()
        {
            proj = new ManagedProj4Projection();
            proj.InternalProjectionParametersString = ManagedProj4Projection.GetDecimalDegreesParametersString();
            proj.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            proj.Open();
            Dictionary<PointShape, double> wellDataPoints = new Dictionary<PointShape, double>();

            StreamReader streamReader = null;
            try
            {
                streamReader = new StreamReader(csvFilePath);
                string headline = streamReader.ReadLine();
                PointShape pointShape;
                while (!streamReader.EndOfStream)
                {
                    string line = streamReader.ReadLine();
                    string[] parts = line.Split(',');
                    pointShape = new PointShape(double.Parse(parts[0]), double.Parse(parts[1]));
                    pointShape = (PointShape)proj.ConvertToExternalProjection(pointShape);
                    wellDataPoints.Add(pointShape,double.Parse(parts[2]));
                }
            }
            finally
            {
                if (streamReader != null) { streamReader.Close(); }
            }
            return wellDataPoints;
        }
    }
}
